@@ -24,13 +24,21 @@
 
 int main()
 {
+    LPCSTR szArchiveName = "e:\\Multimedia\\MPQs\\1995 - Test MPQs\\MPQ_2016_v1_123.w3x";
     HANDLE hMpq = NULL;
+    HANDLE hFile = NULL;
+    BYTE Buffer[0x100];
+    DWORD dwBytesRead = 0;
 
-    _asm int 3;
-    StormOpenFileEx(NULL, NULL, 0, NULL);
+    if(StormOpenArchive(szArchiveName, 0, 0, &hMpq))
+    {                             
+        if(StormOpenFileEx(hMpq, "war3map.j", 0, &hFile))
+        {
+            dwBytesRead = StormGetFileSize(hFile, NULL);
+            StormReadFile(hFile, Buffer, sizeof(Buffer), &dwBytesRead, NULL); 
+            StormCloseFile(hFile);
+        }
 
-    if(StormOpenArchive("E:\\Multimedia\\MPQs\\1995 - Test MPQs\\MPQ_2014_v1_ProtectedMap_Spazzler3.w3x", 0, 0, &hMpq))
-    {
         StormCloseArchive(hMpq);
     }
 
